@@ -1,5 +1,5 @@
 from typing import List, Dict, Optional
-import time, json
+import time, json, traceback
 
 from selenium_firefox.firefox import Firefox, By, Keys
 from kcu import rand
@@ -17,8 +17,7 @@ class Pinterest:
         port: Optional[int] = None
     ):
         self.browser = Firefox(cookies_folder_path, extensions_folder_path, host=host, port=port)
-        self.url_creator = UrlCreator()
-        
+
         try:
             self.browser.get(PT_URL)
             time.sleep(1.5)
@@ -33,15 +32,15 @@ class Pinterest:
                 self.browser.get(PT_URL)
                 time.sleep(1.5)
                 self.browser.save_cookies()
-        except Exception as e:
-            print(e)
+        except:
+            traceback.print_exc()
             self.browser.driver.quit()
 
             raise
 
     def follow(self, user_name: str) -> bool:
         try:
-            self.browser.get(self.url_creator.user_url(user_name))
+            self.browser.get(UrlCreator.user_url(user_name))
             rand.sleep(0.5, 1)
 
             follow_container = self.browser.find(By.XPATH, "//div[contains(@data-test-id, 'user-follow-button')]", timeout=2.5)
@@ -73,38 +72,27 @@ class Pinterest:
                 user_updated = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]")
 
                 return user_updated.text == "Following"
-
-        except Exception as e:
-            print(e)
+        except:
+            traceback.print_exc()
 
             return False
 
     def unfollow(self, user_name: str) -> bool:
         try:
-            self.browser.get(self.url_creator.user_url(user_name))
-            rand.sleep(0.5, 1)
+            self.browser.get(UrlCreator.user_url(user_name))
+            rand.sleep(1.5, 2)
             user_container = self.browser.find(By.XPATH, "//div[contains(@data-test-id, 'user-unfollow-button')]", timeout=2.5)
 
             if user_container is not None:
-                print('x')
-                user_elements = self.browser.find_all(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj tg7 mWe')]", user_container)
+                user_element = self.browser.find_by('button', attributes={'class':'RCK Hsu USg Vxj aZc Zr3 hA- GmH adn a_A gpV hNT iyn BG7 NTm KhY', 'type':'button'})
 
-                for user_element in user_elements:
-                    if user_element.text == "Following":
-                        user_element.click()
-                        rand.sleep(0.5, 1)
-                    else:
-                        return False
+                if user_element is not None:
+                    user_element.click()
+                else:
+                    return False
                 
-                users_updated = self.browser.find_all(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]")
-
-                for elem in users_updated:
-                    if elem.text == "Follow":
-                        print('user.text', elem.text)
-
-                        return True
-
-            elif user_container is None:
+                return self.browser.find_by('button', attributes={'class': 'RCK Hsu USg Vxj aZc Zr3 hA- GmH adn Il7 Jrn hNT iyn BG7 NTm KhY', 'type': 'button'}) is not None
+            else:
                 user = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]")
 
                 if user.text == "Following":
@@ -114,15 +102,14 @@ class Pinterest:
                 user_updated = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]")
 
                 return user_updated.text == "Follow"     
-
-        except Exception as e:
-            print(e)
+        except:
+            traceback.print_exc()
 
             return False
 
-    def repin(self, pin_id: str, board_name: str) -> bool:
+    def repin(self, pin_id: str, board_name: str, needs_repin_id: bool=False) -> (bool, Optional[str]):
         try:
-            self.browser.get(self.url_creator.pin_url(pin_id))
+            self.browser.get(UrlCreator.pin_url(pin_id))
             rand.sleep(0.7, 1.2)
 
             board_dropdown = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc _yT pBj DrD IZT swG z-6')]", timeout=3)
@@ -131,7 +118,12 @@ class Pinterest:
                 board_dropdown.click()
                 rand.sleep(0.1, 0.5)
             else:
-                return self.__create_and_save_to_board(board_name)
+                if self.__create_and_save_to_board(board_name) and needs_repin_id:
+                    return True, self.__get_link_to_repinned_post()
+                elif self.__create_and_save_to_board(board_name) and needs_repin_id is False:
+                    return True, None
+
+                return False, None
 
             boards = self.browser.find_all(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj DrD IZT mWe z-6')]", timeout=5)
 
@@ -139,6 +131,7 @@ class Pinterest:
                 if board.text == board_name:
                     board.click()
                     rand.sleep(0.2,0.5)
+
                     break
             else:
                 self.browser.find(By.XPATH, "//div[contains(@class, 'rDA wzk zI7 iyn Hsu')]").click() # create board button
@@ -146,19 +139,24 @@ class Pinterest:
                 text_tag.send_keys(board_name)
                 rand.sleep(0.5, 1)
                 self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg Vxj aZc Zr3 hA- GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]").click() # create_button
-
-                return True
+                
+                if needs_repin_id:
+                    return True, self.__get_link_to_repinned_post()
+                else:
+                    return self.browser.find(By.XPATH, "//div[contains(@class, 'Eqh Shl s7I zI7 iyn Hsu')]") is not None, None
 
             rand.sleep(0.5, 1)
 
-            return self.browser.find(By.XPATH, "//div[contains(@class, 'Eqh Shl s7I zI7 iyn Hsu')]") is not None
-        
-        except Exception as e:
-            print(e)
+            if needs_repin_id:
+                return self.browser.find(By.XPATH, "//div[contains(@class, 'Eqh Shl s7I zI7 iyn Hsu')]") is not None, self.            __get_link_to_repinned_post()
+            else:
+                return self.browser.find(By.XPATH, "//div[contains(@class, 'Eqh Shl s7I zI7 iyn Hsu')]") is not None, None
+        except:
+            traceback.print_exc()
 
-            return False
+            return False, None
 
-    def get_link_to_repinned_post(self) -> str:
+    def __get_link_to_repinned_post(self) -> Optional[str]:
         try:
             saved_to_button = self.browser.find(By.XPATH, "//div[contains(@class, 'Shl ujU zI7 iyn Hsu')]", timeout=3)
             full_link = self.browser.find(By.CSS_SELECTOR, 'a', saved_to_button).get_attribute('href')
@@ -170,11 +168,9 @@ class Pinterest:
             pin_id = self.browser.find(By.XPATH, "//div[contains(@data-test-id, 'pin')]", latest_image_box).get_attribute('data-test-pin-id')
             rand.sleep(0.1, 0.3)
 
-            if pin_id is not None:
-                return pin_id
-        
-        except Exception as e:
-            print(e)
+            return pin_id
+        except:
+            traceback.print_exc()
 
             return None
     
@@ -189,25 +185,25 @@ class Pinterest:
             self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg Vxj aZc Zr3 hA- GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]").click() # create_button
             rand.sleep(1, 1.5)
 
-            return self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg Vxj aZc Zr3 hA- GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]") is None
-        
-        except Exception as e:
-            print(e)
+            return self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg Vxj aZc Zr3 hA- GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]", timeout=3) is None
+        except:
+            traceback.print_exc()
 
             return False
         
-    def get_board_followers(self, 
+    def get_board_followers(
+        self, 
         user_name: str,
         board_name: str,
         ignored_users: List[str],
         number_of_users_to_follow, 
         full_board_url: str = None
-    ) -> tuple:
+    ) -> Optional[(List[str], List[str])]:
         try:
             if full_board_url is not None:
                 self.browser.get(full_board_url)
             else:
-                self.browser.get(self.url_creator.board_url(user_name, board_name))
+                self.browser.get(UrlCreator.board_url(user_name, board_name))
 
             rand.sleep(1, 1.5)
             followers_container = self.browser.find_all(By.XPATH, "//span[contains(@class, 'tBJ dyH iFc yTZ pBj DrD IZT mWe')]")
@@ -221,23 +217,29 @@ class Pinterest:
             final_users = []
 
             while number_of_users_to_follow >= saved_users:
-                users_list = self.browser.find_all(By.XPATH, "//div[contains(@class, 'Module User hasText thumb medium')]")
-                users_length_before = len(final_users)
+                try:
+                    users_list = self.browser.find_all(By.XPATH, "//div[contains(@class, 'Module User hasText thumb medium')]")
+                    users_length_before = len(final_users)
 
-                for user_container in users_list:
-                    user_url = self.browser.find(By.CSS_SELECTOR, 'a', user_container).get_attribute('href')
-                    user_name = user_url.split('.com/')[1].split('/')[0]
+                    for user_container in users_list:
+                        try:
+                            user_url = self.browser.find(By.CSS_SELECTOR, 'a', user_container).get_attribute('href')
+                            user_name = user_url.split('.com/')[1].split('/')[0]
 
-                    if user_name in ignored_users:
-                        continue
+                            if user_name in ignored_users:
+                                continue
 
-                    ignored_users.append(user_name)
-                    final_users.append(user_name)
-                    saved_users += 1
-                    print(saved_users, ':', user_name)
+                            ignored_users.append(user_name)
+                            final_users.append(user_name)
+                            saved_users += 1
+                            print(saved_users, ':', user_name)
 
-                    if saved_users == number_of_users_to_follow:
-                        return (final_users, ignored_users)
+                            if saved_users == number_of_users_to_follow:
+                                return (final_users, ignored_users)
+                        except:
+                            traceback.print_exc()
+                except:
+                    traceback.print_exc()
                 
                 users_length_after = len(final_users)
                 see_more_button = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj tg7 mWe')]", timeout=1.5)
@@ -248,14 +250,14 @@ class Pinterest:
                 see_more_button.click()
                 rand.sleep(1, 1.5)
 
-        except Exception as e:
-            print(e)
+        except:
+            traceback.print_exc()
 
             return None
 
-    def search_pinterest_boards(self, search_term: str, number_of_boards_to_get: int=35) -> Optional[List[tuple]]:
+    def search_pinterest_boards(self, search_term: str, number_of_boards_to_get: int=35) -> Optional[List[(str, str)]]:
         try:
-            self.browser.get(self.url_creator.search_board_url(search_term))
+            self.browser.get(UrlCreator.search_board_url(search_term))
             rand.sleep(1, 1.5)
 
             if self.browser.find(By.XPATH, "//div[contains(@class, 'noResults')]"):
@@ -269,99 +271,102 @@ class Pinterest:
                 before_scroll = self.browser.current_page_offset_y()
 
                 for board_name_element in board_names_container:
-                    full_board_url = self.browser.find(By.CSS_SELECTOR, 'a', board_name_element).get_attribute('href')
-                    board_info = full_board_url.split('.com/')[1]
-                    user_name = board_info.split('/')[0]
-                    board_name = board_info.split('/')[1]
+                    try:
+                        full_board_url = self.browser.find(By.CSS_SELECTOR, 'a', board_name_element).get_attribute('href')
+                        board_info = full_board_url.split('.com/')[1]
+                        user_name = board_info.split('/')[0]
+                        board_name = board_info.split('/')[1]
 
-                    if (user_name, board_name) in board_urls:
-                        continue
+                        if (user_name, board_name) in board_urls:
+                            continue
 
-                    board_urls.append((user_name, board_name))
-                    number_of_saved_boards += 1
+                        board_urls.append((user_name, board_name))
+                        number_of_saved_boards += 1
+
+                        if number_of_boards_to_get == number_of_saved_boards:
+                            return board_urls
+                    except:
+                        traceback.print_exc()
 
-                    if number_of_boards_to_get == number_of_saved_boards:
-                        return board_urls
-                    
                 self.browser.scroll(1000)
                 rand.sleep(0.5, 1.5)
                 after_scroll = self.browser.current_page_offset_y()
 
                 if after_scroll == before_scroll:
                     return board_urls
-
-        except Exception as e:
-            print(e)
+        except:
+            traceback.print_exc()
 
             return None
 
     def get_pins_from_home_feed(self) -> Optional[List[str]]:
         try:
-            self.browser.get(self.url_creator.home_feed_url())
+            self.browser.get(UrlCreator.home_feed_url)
             rand.sleep(1, 1.5)
 
             home_pins = []
             home_pin_containers = self.browser.find_all(By.XPATH, "//div[contains(@class, 'Yl- MIw Hb7')]")
 
             for pin in home_pin_containers:
-                full_url = self.browser.find(By.CSS_SELECTOR, 'a', pin).get_attribute('href')
+                try:
+                    full_url = self.browser.find(By.CSS_SELECTOR, 'a', pin).get_attribute('href')
 
-                if 'pinterest.com' not in full_url:
-                    continue
-                
-                if 'pin/' in full_url:
-                    pin_id = full_url.split('pin/')[1]
-                    home_pins.append(pin_id)
+                    if 'pinterest.com' not in full_url:
+                        continue
+                    
+                    if 'pin/' in full_url:
+                        pin_id = full_url.split('pin/')[1]
+                        home_pins.append(pin_id)
+                except:
+                    traceback.print_exc()
                     
             return home_pins
-        
-        except Exception as e:
-            print(e)
+        except:
+            traceback.print_exc()
 
             return None
 
-    def post_pin(self, 
+    def post_pin(
+        self, 
         file_path: str,
         title_text: str,
         board_name: str,
-        about_text: str = None,
-        destination_link_text: str = None
-        ) -> str:
+        about_text: Optional[str] = None,
+        destination_link_text: Optional[str] = None
+    ) -> Optional[str]:
         try:
-            self.browser.get(self.url_creator.pin_builder_url())
+            self.browser.get(UrlCreator.pin_builder_url)
             rand.sleep(1, 1.5)
-
-            select_board_button = self.browser.find(By.XPATH, "/html/body/div[1]/div[1]/div[3]/div/div/div/div[2]/div/div/div/div/div/div/div/div[1]/div/div[2]/div/div/div")
-            if select_board_button is not None:
-                select_board_button.click()
-                rand.sleep(0.1, 0.5)
+            select_board_button = self.browser.find_by('button', attributes={'data-test-id': 'board-dropdown-select-button'})
+            select_board_button.click()
+            rand.sleep(0.1, 0.5)
             
             boards = self.browser.find_all(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj DrD IZT mWe z-6')]") 
 
-            if boards is not None: 
-                for board in boards:
-                    if board.text == board_name:
-                        board.click()
-                        rand.sleep(0.1, 0.5)
+            for board in boards:
+                if board.text == board_name:
+                    board.click()
+                    rand.sleep(0.1, 0.5)
 
-                        break
-                else:
-                    dropdown_boards = self.browser.find(By.XPATH, "//div[contains(@class, 'DUt qJc sLG zI7 iyn Hsu')]")
-                    create_board = self.browser.find(By.XPATH, "//div[contains(@class, 'rDA wzk zI7 iyn Hsu')]", dropdown_boards)
+                    break
+            else:
+                dropdown_boards = self.browser.find(By.XPATH, "//div[contains(@class, 'DUt qJc sLG zI7 iyn Hsu')]")
+                create_board = self.browser.find(By.XPATH, "//div[contains(@class, 'rDA wzk zI7 iyn Hsu')]", dropdown_boards)
 
-                    if create_board is not None:
-                        create_board.click()
-                        rand.sleep(0.1, 0.5)
-                    
-                    text_tag = self.browser.find(By.XPATH, "//input[contains(@id, 'boardEditName')]")
-                    text_tag.send_keys(board_name)
-                    rand.sleep(0.5, 1)
-                    self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg F10 xD4 fZz hUC GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]").click()
-                    rand.sleep(0.5, 1)
-                    just_created_board_save_bttn = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh DrD IZT mWe')]")
-                    just_created_board_save_bttn.click()
+                if create_board is not None:
+                    create_board.click()
+                    rand.sleep(0.1, 0.5)
+                
+                text_tag = self.browser.find(By.XPATH, "//input[contains(@id, 'boardEditName')]")
+                text_tag.send_keys(board_name)
+                rand.sleep(0.5, 1)
+                self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg F10 xD4 fZz hUC GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]").click()
+                rand.sleep(0.5, 1)
+                just_created_board_save_bttn = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh DrD IZT mWe')]")
+                just_created_board_save_bttn.click()
 
             title_box = self.browser.find(By.XPATH, "//div[contains(@class, 'CDp xcv L4E zI7 iyn Hsu')]")
+
             if title_box is not None:
                 title = self.browser.find(By.CSS_SELECTOR, "textarea", title_box)
                 title.send_keys(title_text)
@@ -369,50 +374,38 @@ class Pinterest:
             
             if about_text is not None:
                 about_box = self.browser.find(By.XPATH, "//div[contains(@class, 'Jea Tte ujU xcv L4E zI7 iyn Hsu')]")
+
                 if about_box is not None:
                     about = self.browser.find(By.CSS_SELECTOR, "textarea", about_box)
                     about.send_keys(about_text)
                     rand.sleep(0.2, 0.5)
 
-
             if destination_link_text is not None:
                 destination_link_box = self.browser.find(By.XPATH, "//div[contains(@data-test-id, 'pin-draft-link')]")
+
                 if destination_link_box is not None:
                     destination_link = self.browser.find(By.CSS_SELECTOR, "textarea", destination_link_box)
                     destination_link.send_keys(destination_link_text)
                     rand.sleep(0.2, 0.5)
             
             image_box = self.browser.find(By.XPATH, "//div[contains(@class, 'DUt XiG zI7 iyn Hsu')]")
+
             if image_box is not None:
                 image = self.browser.find(By.CSS_SELECTOR, 'input', image_box)
                 image.send_keys(file_path)
                 rand.sleep(1, 1.5)
             
             save_button = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh DrD IZT mWe')]")
+
             if save_button is not None:
                 save_button.click()
             
             rand.sleep(0.1, 0.5)
             see_it_now = self.browser.find(By.XPATH, "//div[contains(@data-test-id, 'seeItNow')]")
             full_pin_url = self.browser.find(By.CSS_SELECTOR, 'a', see_it_now).get_attribute('href')
-            pin_id = full_pin_url.split('pin/')[1]
 
-            return pin_id
+            return full_pin_url.split('pin/')[1]
+        except:
+            traceback.print_exc()
 
-        except Exception as e:
-            print(e)
-
-            return None
-
-    def disable_pop_up(self):
-        try:
-            not_now = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]")
-            if not_now.text == "Not now":
-                not_now.click()
-
-            return True
-        
-        except Exception as e:
-            print(e)
-
-            return False
+            return None
\ No newline at end of file
